@@ -262,7 +262,7 @@ proc vMAXAddSystemMenu {fr text} {
     set xfile [file join $maxima_priv(maxima_verpkgdatadir) xmaxima html xmaxima.html]
     if {[file isfile $xfile]} {
 	set xstate normal
-	if {$tcl_platform(platform) == "windows"} {
+	if {$tcl_platform(platform) == "windows" && $tcl_platform(osVersion) < 5 } {
 	    # decodeURL is broken and needs fixing
 	    # This is a workaround
 	    set xfile [file attrib $xfile -shortname]
@@ -358,8 +358,7 @@ proc SetPlotFormat { text } {
     global maxima_default
     
     if { $maxima_default(plotwindow) == "embedded" } {
-	#sendMaxima $text "set_plot_option(\[plot_format,openmath\])\$\n"
-	sendMaxima $text ":lisp-quiet (prog2 (\$set_plot_option '((mlist simp) \$plot_format \$openmath)) nil) \n"
+	sendMaxima $text ":lisp-quiet (prog2 (\$set_plot_option '((mlist simp) \$plot_format \$xmaxima)) nil) \n"
     }
     
 }
