@@ -10,7 +10,7 @@
 (defvar *varlist* nil)
 (defvar *genvar* nil)
 
-(defun polynomialp (x)
+(defun affine-polynomialp (x)
   (cond  ((numberp x))
 	 ((atom x) nil)
 	 ((and (atom (car x)) (symbolp (car x)) (get (car x) 'disrep)))
@@ -22,14 +22,14 @@
 	       (numberp (cdr expr)))
 	  (cond ((zerop (car expr))0)
 		(t (list '(rat simp) (car expr) (cdr expr)))))
-	 ((polynomialp expr)(cons (list 'mrat 'simp *varlist* *genvar*) (cons expr 1)))
+	 ((affine-polynomialp expr)(cons (list 'mrat 'simp *varlist* *genvar*) (cons expr 1)))
 	 ((rational-functionp expr)(cons (list 'mrat 'simp *varlist* *genvar* ) expr ))
 	 (t expr)))
 
 (defun $zerop ( n &aux type-of-n tem )
   (cond ((numberp n) (zerop n))
 	((atom n) nil)
-	((polynomialp n) nil)
+	((affine-polynomialp n) nil)
 	((rational-functionp n)(rzerop   n))
 	(($bfloatp n) (eql (car n) 0))
 	(t (setf type-of-n (caar n))
@@ -40,8 +40,8 @@
 (defun rational-functionp (x)
   (cond ((numberp x))
 	((atom x) nil)
-	(t (and (polynomialp (car x))
-		(polynomialp (cdr x))))))
+	(t (and (affine-polynomialp (car x))
+		(affine-polynomialp (cdr x))))))
 
 (defun psublis (a-list denom poly &key degree vars-to-sub)
   "does a general sublis : a-list of form (list (cons old-var repl-poly)....)
@@ -49,7 +49,7 @@
   (let ((tem (cond (vars-to-sub vars-to-sub)
 		   (t (loop for v in a-list collecting (car v)))))
 	deg)
-    (cond ((polynomialp poly)
+    (cond ((affine-polynomialp poly)
 	   (cond (degree (setq deg degree))
 		 (t (setq deg (poly-degree poly tem))))
 	   (psublis1 a-list denom poly
@@ -155,7 +155,7 @@
 		   (:polynomial (,rat-op ,x (cons ,y 1)  ,@ rat-switch))
 		   (:rational-function (,rat-op ,x ,y ,@ rat-switch))))
 		(otherwise (merror "unknown arg"))))
-	(cond ((polynomialp answer) answer)
+	(cond ((affine-polynomialp answer) answer)
 	      ((rational-functionp answer)
 	       (cond ((eq 1 (cdr answer))(car answer))
 		     (t answer)))
@@ -226,7 +226,7 @@
   (cond ( $display2d (setq disp 'displa)))
   (let ((*standard-output* stream))
     (setq answ(cond ((numberp expr) (funcall disp expr))
-		    ((polynomialp expr)(funcall disp (cons  *fake-rat* (cons expr 1))))
+		    ((affine-polynomialp expr)(funcall disp (cons  *fake-rat* (cons expr 1))))
 		    ((rational-functionp expr)(funcall disp (cons *fake-rat* expr)))
 		    (t (funcall disp expr))))
     (cond (answ (format t "~A" answ)))))
@@ -235,7 +235,7 @@
   (cond ((symbolp expr) (setq .expr. expr))
 	(t (setq .expr. '.expr.)))
   (setq form
-	`(cond ((polynomialp  , .expr.) (setq ,num , .expr. , den  1))
+	`(cond ((affine-polynomialp  , .expr.) (setq ,num , .expr. , den  1))
 	       ((rational-functionp , .expr.) (setq ,num (num , .expr.) ,den (denom , .expr.)))
 	       (t (fsignal "expr is supposed to be poly or rational-functionp"))))
   (cond ((symbolp expr)
