@@ -1,6 +1,6 @@
 ;; Author Barton Willis
 ;; University of Nebraska at Kearney
-;; Copyright (C) 2004, Barton Willis
+;; Copyright (C) 2004, 2009, Barton Willis
 
 ;; Brief Description: Maxima code for linear homogeneous second order
 ;; differential equations.
@@ -12,13 +12,13 @@
 ;; Maxima odelin has NO WARRANTY, not even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
-($put '$odelin 19 '$version)
+($put '$odelin 20 '$version)
 
 ;; [DB 2007-04-29] I have added some comments to generic-de-solver and 
 ;; subsidiary routines.  They are just my interpretation of the code.
 
 ;; The functions mtimesp and mexptp are either missing from 
-;; commerical macsyma, or they have different names. For commerical
+;; commercial macsyma, or they have different names. For commercial
 ;; macsyma, here are the definitions.
 
 #+kcl
@@ -39,32 +39,11 @@
     ($load "extrabessel")
     ($load "lazysolver")
     ($load "gauss")
+    ($load "functs") ;; wronskian
     ($load "odeutils"))
 
-(defmvar $de_solver_is_loquacious nil)
-
 (defun ode-polynomialp (p x)
-  (setq p ($ratdisrep p))
-  (or
-   ($freeof x p)
-   (like p x)
-   (and (or (mtimesp p) (mplusp p)) 
-	(every #'(lambda (s) (ode-polynomialp s x)) (margs p)))
-   (and (mexptp p) (ode-polynomialp (car (margs p)) x) 
-	(integerp (cadr (margs p)))
-	(> (cadr (margs p)) 0))))
-   
-(defun $okay (de y x)
-  (let ((okay t))
-  (setq de (require-linear-homogeneous-de de y x))
-  (dolist (p de)
-    (setq okay (and okay (ode-polynomialp ($ratnumer p) x) 
-		    (ode-polynomialp ($ratnumer p) x))))
-  okay))
-
-(defun $rerat (dirtyrat)
-  (let ((varlist nil) (genvar nil))
-  ($rat ($ratdisrep dirtyrat))))
+  ($polynomialp p `((mlist) ,x) `((lambda) ((mlist) s) ($freeof ,x s))))
    
 (defun require-linear-homogeneous-de (de y x)
   (setq y (require-symbol y "$odelin"))
@@ -98,14 +77,12 @@
 (defun $odelin (de y x)
   (let ((cfs (require-linear-homogeneous-de de y x)) (n))
     (setq n (length cfs))
-    (cond ((= n 2) ($expand (odelin-order-one cfs x) 0))
-	  ((= n 3) ($expand (odelin-order-two cfs x) 0))
+    (cond ((= n 2) (odelin-order-one cfs x))
+	  ((= n 3) (odelin-order-two cfs x))
 	  (t (merror "'odelin' doesn't handle DEs with order ~:M" (- n 1))))))
 
 (defun odelin-order-one (cfs x)
-  (let ((p1 (nth 1 cfs))
-	(p0 (nth 0 cfs)))
-    (fss-cleanup `(($set) ,(power '$%e ($integrate (div p0 p1) x))) x)))
+  (fss-cleanup (take '($set) ($exp ($integrate (div (car cfs) (cadr cfs)) x))) x))
 
 (defun expunge-const-factors (e x)
   (let ((acc 1))
@@ -119,12 +96,9 @@
 ;; Cleanup a fundamental solution set (FSS). 
   
 (defun fss-cleanup (fss x)
-  (let ((nfss) (k 0) ($radexpand nil) ($ratsimpexpons t))
-    (while (and (< k 5) (not (like fss (setq nfss (mbag-map #'$radcan fss)))))
-      (incf k)
-      (setq fss nfss))
-    (mbag-map #'(lambda (s) (expunge-const-factors s x)) fss)))
-
+  (let ((nfss (mbag-map #'(lambda (s) (expunge-const-factors ($radcan s) x)) fss)))
+    (if (like nfss fss) fss (fss-cleanup nfss x))))
+       
 (defun post-check-cleanup (fss)
   (setq fss ($substitute '%bessel_j '$fbessel_j fss))
   ($substitute '%bessel_y '$fbessel_y fss))
@@ -133,16 +107,15 @@
   (let ((p0 (nth 0 cfs))
 	(p1 (nth 1 cfs))
 	(p2 (nth 2 cfs))
-	(p) (m) (sol)
+	(p) (m) (sol nil)
 	(ode-methods (list 
 		      'ode-solve-by-factoring
 		      'bessel-de-solver
 		      'hypergeo01-de-solver
 		      'spherodialwave-de-solver
 		      'bessel-sqrt-de-solver
-		      'hypergeo21-de-solver))
-	($radexpand nil))
-    
+		      'hypergeo21-de-solver)))
+
     (setq p1 (div p1 p2))
     (setq p0 (div p0 p2))
 
@@ -153,14 +126,15 @@
 	(setq ode-methods nil))
       
     (setq p (add p0 (div ($diff p1 x) -2) (mul p1 p1 (div -1 4))))
-    (setq p ($rat p))
-    (setq m (power '$%e (simplify ($integrate (div p1 -2) x))))
+    (setq m ($exp ($integrate (div p1 -2) x)))
        
-    (dolist (method ode-methods)
-      (setq sol (funcall method p x))
-      (if ($setp sol) (setq sol (fss-cleanup 
-				 (mbag-map #'(lambda (s) (mul m s)) sol) x)))
-      (if (check-fss cfs sol x) (return (post-check-cleanup sol))))))
+    (while (and (not sol) ode-methods)
+      (setq sol (funcall (pop ode-methods) p x))
+      (if ($setp sol)
+	  (progn
+	    (setq sol (fss-cleanup (mbag-map #'(lambda (s) (mul m s)) sol) x))
+	    (setq sol (if (check-fss cfs sol x) (post-check-cleanup sol) nil)))))
+    sol))
    
 (defprop unk simp-unk operators)
 
@@ -169,7 +143,7 @@
   (merror "Maxima doesn't know the derivative of ~:M with respect the ~:M argument" (nth 2 x) (nth 1 x)))
 	  		 
 (defun check-de-sol (cfs sol x)
-  (let ((zip 0) ($gcd '$spmod) ($algebraic t) ($ratsimpexpons t))
+  (let ((zip 0))
     (dolist (cf cfs)
       (setq zip (add zip (mul cf sol)))
       (setq sol ($diff sol x)))
@@ -180,32 +154,11 @@
      (mtell "should vanish, but it does not ~:M~%" zip))))
 
 (defun check-fss (cfs fss x)
-  (let (($gcd '$spmod) ($algebraic t))
-    (and 
-     ($setp fss)
-     (= (length cfs) (length fss))
-     (every #'(lambda (s) (check-de-sol cfs s x)) (cdr fss))
-     (not (like 0 ($xwronskian fss x))))))
-
-(defun check-number-of-args (n arg f)
-  (if (not (= n (length arg)))
-      (merror "Function ~:M requires ~:M arguments; found ~:M arguments"
-	      f n (+ n (length arg)))))
-  
-;; Return the wronskian of the list or set of expressions s. 
-
-(defun $xwronskian (&rest arg)
-  (check-number-of-args 2 arg "$xwronskian")
-  (let ((mat) (dim)
-	(s (require-list-or-set (nth 0 arg) "$xwronskian"))
-	(x (require-symbol (nth 1 arg) "$xwronskian")))
-    (setq s `((mlist) ,@s))
-    (setq x (require-symbol (nth 1 arg) "$xwronskian"))
-    (setq dim ($length s))
-    (dotimes (i dim)
-      (push s mat)
-      (setq s ($diff s x)))
-    ($radcan ($determinant `(($matrix) ,@mat)))))
+  (and 
+   ($setp fss)
+   (= (length cfs) (length fss))
+   (every #'(lambda (s) (check-de-sol cfs s x)) (cdr fss))
+   (not (like 0 ($radcan ($determinant (mfuncall '$wronskian ($listify fss) x)))))))
 
 ;; Return a polynomial in x with degree deg and a list of its
 ;; coefficients. Each coefficient of the polynomial is a gensym.
@@ -221,7 +174,7 @@
       (setq z (mul z x)))))
 
 (defun polycfs-to-eqs (p x)
-  (let ((n) (eqs) ($gcd '$spmod) ($algebraic t) ($ratfac nil) ($ratprint nil))
+  (let ((n) (eqs) ($ratfac nil) ($ratprint nil))
     (setq p ($rat p x))
     (setq n (+ 1 ($hipow p x)))
     (dotimes (i n eqs)
@@ -235,7 +188,6 @@
 ;; and f = mu * integrate(1/mu^2,x).
 
 (defun ode-solve-by-factoring (r x)
-  (if $de_solver_is_loquacious (mtell "...trying factor method~%"))
   (let ((a) (mu) ($radexpand nil))
     (cond ((setq a (factor-differential-op r x))
 	   (setq a ($radcan a)) ;; x*(x+1)*'DIFF(y,x,2)+(3*x+2)*'DIFF(y,x,1)+y
@@ -255,7 +207,7 @@
        (m ($hipow q x))
        (l ($hipow p x))
        (w) (zip) (n) (vars) (sol)
-       ($gcd '$spmod) ($algebraic t) ($ratfac nil))
+       ($ratfac nil))
   
     (setq m ($totaldisrep m))
     (setq l ($totaldisrep l))
@@ -290,8 +242,7 @@
 	   nil))))
 
 (defun polynomial-filter (p x f)
-  (let (($gcd '$spmod) ($algebraic t) ($ratfac nil) 
-	($ratprint nil) ($radexpand nil))
+  (let (($ratfac nil) ($ratprint nil))
     (setq p (sratsimp p)) ;; Get rid of terms like sqr(5)^2, %i^2...
     (setq p ($mysqfr p x))
     (setq p (if (mtimesp p) (margs p) (list p)))
@@ -308,7 +259,7 @@
 	(setq q (mult q (power pj n)))))))
   
 (defun ratfun-degree (q x)
-  (let (($gcd '$spmod) ($algebraic t) ($ratfac nil))
+  (let (($ratfac nil))
     (setq q ($rat q x))
     (- ($hipow ($ratnumer q) x) ($hipow ($ratdenom q) x))))
 
@@ -327,8 +278,7 @@
 ;; routine don't get solved. 
 
 (defun easy-eqs (cnd s x)
-  (let ((acc) (n) ($gcd '$spmod) ($algebraic t) 
-	($programmode t) 
+  (let ((acc) (n) ($programmode t) 
 	($globalsolve nil) ($solveexplicit t) ($solveradcan nil))
     
     (setq s (polynomial-filter s x #'(lambda (n) (min 1 n))))
@@ -428,8 +378,7 @@
 ;;
 (defun generic-de-solver (v x params denom-filter degree-bound de-cnd)
   (setq v ($rat v x))
-  (let ((s) (q) (p) (n) (unks) (nz) (eqs) (cnd) (sol) (xeqs)
-	($gcd '$spmod) ($algebraic t) ($ratfac nil))
+  (let ((s) (q) (p) (n) (unks) (nz) (eqs) (cnd) (sol) (xeqs) ($ratfac nil))
 
     (setq s ($ratdenom v))
     (setq q (polynomial-filter s x denom-filter))
@@ -467,8 +416,7 @@
 ;;    We return (-1) times equation above
 ;;
 (defun get-de-cnd (xi vh v x)
-  (let ((dxi) (ddxi) (dddxi) (dxi^2) ($gcd '$spmod) ($algebraic t) 
-	($ratfac nil) ($ratprint nil))
+  (let ((dxi) (ddxi) (dddxi) (dxi^2) ($ratfac nil) ($ratprint nil))
     
     (setq xi ($ratdisrep xi))
     (setq dxi ($diff xi x))
@@ -502,9 +450,6 @@
   (ceiling (+ 1 ($hipow q x) (/ (max (ratfun-degree v x) -2) 2))))
 
 (defun bessel-de-solver (v x)
-  (if $de_solver_is_loquacious
-      (mtell "...trying the Bessel solver~%"))
-
   (let* 
       ((mu (gensym)) (z) (m) ($radexpand nil)
        (xi (generic-de-solver v x 
@@ -545,9 +490,6 @@
   (+ 2 ($hipow q x) (max -2 (ratfun-degree v x))))
 
 (defun bessel-sqrt-de-solver (v x)
-  (if $de_solver_is_loquacious
-      (mtell "...trying the square root Bessel solver~%"))
-
   (let*  ((mu (gensym)) (z) (m) ($radexpand nil)
 	  (xi (generic-de-solver v x 
 				 (list mu)
@@ -587,7 +529,6 @@
   (ceiling (+ 1 ($hipow q x) (/ (max (ratfun-degree v x) -2) 2))))
 
 (defun hypergeo01-de-solver (v x)
-  (if $de_solver_is_loquacious (mtell "...trying the F01 solver~%"))
   (let* ((a (gensym)) (b (gensym)) (m) (z) ($radexpand nil)
 	 (xi (generic-de-solver v x 
 				(list a b)
@@ -629,7 +570,7 @@
 ;;
 ;; Additional parameters to be determined are [b,c,q]
 ;;
-;; FIXME: [DB 2007-04-29] Need to clarify definition of the spherodial
+;; FIXME: [DB 2007-04-29] Need to clarify definition of the spheroidal
 ;; wave functions as I can't find a reference that exactly matches.
 ;; The values of a1 and a0 above:
 ;; - give an ODE that is satisfied by spherodialwave_a and spherodialwave_b, 
@@ -642,7 +583,6 @@
   (ceiling (+ 1 ($hipow q x) (/ (max (ratfun-degree v x) -2) 2))))
 
 (defun spherodialwave-de-solver (v x)
-  (if $de_solver_is_loquacious (mtell "...trying the spherodial wave solver~%"))
   (let* ((b (gensym)) (c (gensym)) (q (gensym)) (m) (z) (x2 (mul x x))
 	 ($radexpand nil)
 	 (xi (generic-de-solver v x 
@@ -700,9 +640,7 @@
   (declare (ignore q v x))
   1)
 
-
 (defun hypergeo21-de-solver (v x)
-  (if $de_solver_is_loquacious (mtell "...trying the 2F1 solver~%"))
   (let* ((a (gensym)) (b (gensym)) (c (gensym)) (m) (z) ($radexpand nil)
 	 (xi (generic-de-solver v x 
 				(list a b c)
