@@ -22,27 +22,48 @@
 ;;******************************************************************************
 ;;				driver 	stage
 ;;******************************************************************************
-;;
-;;		the following throw labels are used
-;;
-;;psex -- for throws on failure to expand
-;;
 
 (defmfun $powerseries (expr var pt)
-  (when (numberp var)
+  (when (and (atom var) (not (symbolp var)))
     (improper-arg-err var '$powerseries))
+  (powerseries expr var pt))
 
-  (if (and (signp e pt) (symbolp var))
-      (seriesexpand* expr)
-      (cond
-        ((signp e pt)
-         (sbstpt expr 'x var var))
-        ((eq pt '$inf)
-         (sbstpt expr (m// 1 'x) var (div* 1 var)))
-        ((eq pt '$minf)
-         (sbstpt expr (m- (m// 1 'x)) var (m- (div* 1 var))))
-        (t
-         (sbstpt expr (m+ 'x pt) var (simplifya (m- var pt) nil))))))
+;; An error that can be raised deep in the bowels of POWERSERIES that we'll
+;; catch and return a noun form.
+(define-condition powerseries-expansion-error (error)
+  ((message :initarg :message :initform nil)))
+
+(defun powerseries-expansion-error (&optional message &rest arguments)
+  (error 'powerseries-expansion-error
+         :message (when message (cons message arguments))))
+
+;; The top-level routine for $powerseries, which calls this function after
+;; spotting invalid arguments.
+(defun powerseries (expr var pt)
+  (handler-case
+      (if (and (signp e pt) (symbolp var))
+          (seriesexpand* expr)
+          (cond
+            ((signp e pt)
+             (sbstpt expr 'x var var))
+            ((eq pt '$inf)
+             (sbstpt expr (m// 1 'x) var (div* 1 var)))
+            ((eq pt '$minf)
+             (sbstpt expr (m- (m// 1 'x)) var (m- (div* 1 var))))
+            (t
+             (sbstpt expr (m+ 'x pt) var (simplifya (m- var pt) nil)))))
+
+    ;; If expansion fails, print a message in verbose mode but then return a
+    ;; noun form.
+    (powerseries-expansion-error (e)
+      (when $verbose
+        (mtell (intl:gettext "Failed to expand ~M with respect to ~M at ~M.~%")
+               expr var pt)
+        (with-slots (message) e
+          (when message
+            (terpri)
+            (apply #'mtell message))))
+      `((%powerseries) ,expr ,var ,pt))))
 
 ;; Return a list of the terms in the expression that are used as integration or
 ;; differentiation variables.
@@ -69,28 +90,24 @@
   (let* ((var (gensym))
          (sub-exp (maxima-substitute (subst var 'x sexp) var1 exp))
          (expanded (seriesexpand* sub-exp)))
-    ;; If we've ended up with diff(foo, var) then we give up and return the
-    ;; original contents (we can't substitute arbitrary expressions for the
-    ;; differentiation / integration variable).
+    ;; If we've ended up with diff(foo, var) then we give up (we can't
+    ;; substitute arbitrary expressions for the differentiation / integration
+    ;; variable).
     (if (memq var (intdiff-vars-in-expr expanded))
-        exp
+        (powerseries-expansion-error
+         (intl:gettext "~
+Couldn't make substitution to evaluate at the given point because the~%~
+power series expansion contained the expansion variable as an~%~
+integration / differentiation variable."))
         (maxima-substitute usexp var expanded))))
 
 (defun seriesexpand* (x)
-  (let (*n *a *m *c
-	   (*index (gensumindex))
-	   ($cauchysum t)
-	   ($ratsimpexpons t) $ratexpand
-	   *infsumsimp *ratexp *trigred *noexpand)
+  (let ((*index (gensumindex)) *n *a *m *c
+        ($cauchysum t) ($ratsimpexpons t)
+        $ratexpand *infsumsimp *ratexp *trigred *noexpand)
     (meval `(($declare) ,*index $integer))
-    (setq x (catch 'psex (sp2expand (seriespass1 x))))
-    (cond ((and x (atom x)) x)
-	  ((and x (not (eq (car x) 'err))) x)
-	  ($verbose
-	   (mtell (intl:gettext "powerseries: unable to expand for the following reason: "))
-	   (cond ((null x) (mtell (intl:gettext "no reason given")) (intl:gettext "Failed to expand"))
-		 (t (cdr x))))
-	  (t (intl:gettext "Failed to expand")))))
+
+    (sp2expand (seriespass1 x))))
 
 (defun out-of (e)
   (let  ((e      (cond ((and (boundp '*var) *var)
@@ -126,7 +143,6 @@
 	((sratp exp var) (ratexp exp))
 	((eq (caar exp) 'mexpt) (sp2expt exp))
 	((oldget (caar exp) 'sp2) (sp2sub (sp2trig exp) (cadr exp)))
-	((poly? exp var) exp)
 	((eq (caar exp) 'mtimes) (m*l (mapcar #'sp2expand (cdr exp))))
 	((eq (caar exp) '%log) (sp2log (cadr exp)))
 	((eq (caar exp) '%derivative) (sp2diff (cadr exp) (cddr exp)))
@@ -142,31 +158,34 @@
 		 *index 0 '$inf))))
 
 (defun sp2sub (s exp)
-  (cond ((smono exp var) (maxima-substitute exp 'sp2var (simplify s)))
-	(t (throw 'psex (list 'err '(mtext)
-			      "Tried to `maxima-substitute' the expansion of  "
-			      (out-of exp)
-			      " into an expansion")))))
+  (unless (smono exp var)
+    (powerseries-expansion-error
+     (intl:gettext "Can't substitute the value~%~M~%~
+                    into another expansion because it isn't a monomial in the~
+                    expansion variable.")
+     (out-of exp)))
+  (maxima-substitute exp 'sp2var (simplify s)))
 
 (defun ratexp (exp)
-  (let (nn* dn* *gcd*)
+  (let (*gcd*)
     (if $verbose
 	(mtell (intl:gettext "powerseries: attempt rational function expansion of~%~M")
 	       (list '(mlabel) nil exp)))
-    (numden exp)
-    (sratexpnd nn* dn*)))
+    (multiple-value-call #'sratexpnd (numden exp))))
 
 (defun sratexpnd (n d)
   (let ((ans (list nil))
         (*splist*)
+        ;; A pattern that matches cc*(c*x^m + a)^n
         (linpat
          '((mtimes) ((coefftt) (cc not-zero-free var))
-                  ((mexpt) ((mplus) ((coeffpt)
-                               (w m1 ((mexpt) (x equal var)
-                                          (m not-zero-free var)))
-                               (c freevar))
-                              ((coeffpp) (a freevar)))
-                         (n not-zero-free var)))))
+           ((mexpt) ((mplus) ((coeffpt)
+                              (w m1 ((mexpt) (x equal var)
+                                     (m not-zero-free var)))
+                              (c freevar))
+                     ((coeffpp) (a freevar)))
+            (n not-zero-free var)))))
+
     (declare (special *splist*))
       (cond ((and (not (equal n 1)) (smono n var))
              (m* n (sratexpnd 1 d)))
@@ -174,8 +193,9 @@
              (cond ((poly? n var)
 		    (m// n d))
 		   ((m1 n linpat)
-		    (m* (srbinexpnd (cdr ans)) (div* 1 d)))
-		   ((throw 'psex nil))))
+		    (m// (srbinexpnd (cdr ans)) d))
+		   (t
+                    (powerseries-expansion-error))))
             ((smonop d var)
              (cond ((mplusp n)
                   (m+l (mapcar #'(lambda (q) (div* q d)) (cdr n))))
@@ -183,33 +203,56 @@
             ((not (equal 1 (setq *gcd* (ggcd (nconc (exlist n) (exlist d))))))
              (sratsubst *gcd* n d))
 
-	    ; rational expansion theorem for distinct roots
+	    ;; Rational expansion theorem for distinct roots. The main point of
+	    ;; the POLY? test here is that it guarantees $HIPOW will give the
+	    ;; correct degrees (in particular, it only allows known non-negative
+	    ;; integer exponents)
 	    ((and (poly? n var)
 		  (poly? d var)
 		  (> ($hipow d var) ($hipow n var))
 		  (has-distinct-nonzero-roots-p d var))
 	     (expand-distinct-roots n d))
 
-            ((and (equal n 1)
-                (prog2 (setq d (let (($ratfac t))
-                              (ratdisrep ($rat (factor d) var))))
-                     (m1 d linpat)))
+            ;; The SRBINEXPND call above dealt with expressions of the form
+            ;; cc*(c*x^m+a)^n. Here, we deal with b/(cc*(c*x^m+a)^n). If you
+            ;; explicitly write a polynomial like that, the simplifier will
+            ;; rewrite it as (..)^(-n) before it gets here, but things like
+            ;; 1/sqrt(x+1) won't have been rewritten successfully, so we catch
+            ;; them here.
+            ((and (free n var)
+                  (prog2 (setq d (let (($ratfac t))
+                                   (ratdisrep ($rat (factor d) var))))
+                      (m1 d linpat)))
+             ;; We had num/den and LINPAT matched den. We need to replace cc
+             ;; with num/cc and n with -n.
+             (setf (cdr (assoc 'n ans)) (m- (cdr (assoc 'n ans)))
+                   (cdr (assoc 'cc ans)) (m// n (cdr (assoc 'cc ans))))
+             (srbinexpnd (cdr ans)))
 
-             ;; negate exponent because pattern matched denominator
-	     (setf (cdadr ans) (mul -1 (cdadr ans)))
-
-             (m// (srbinexpnd (cdr ans)) (cdr (assoc 'cc (cdr ans) :test #'eq))))
             (t
-             (and *ratexp (throw 'psex nil))
-             (if (not (eq (caar d) 'mtimes)) 
-		 (ratexand1 n d)
-	       (do ((p (cdr d) (cdr p)))	; denom is a product
-               ((null p) (ratexand1 n d))
-		 ; look for power of var (zero root) as term of denom
-               (cond ((or (eq (car p) var)
-                        (and (mexptp (car p)) (eq (cadaar p) var)))
-                    (return (m* (sratexpnd n (meval (div* d (car p))))
-				    (list '(mexpt) (car p) -1)))))))))))
+             ;; *RATEXP is set by RATEXPAND1, which we call to do the general
+             ;; expansion below. This check makes sure we can't end up recursing
+             ;; infinitely.
+             (when *ratexp
+               (powerseries-expansion-error))
+
+             ;; Look for a power of var (a zero root) as a term. We already know
+             ;; that d isn't a monomial, so we can only have a zero root if d is
+             ;; a product. We also know that d is not atomic because otherwise
+             ;; we'd have taken the (free d var) clause above.
+             (let ((zero-root
+                    (and (eq (caar d) 'mtimes)
+                         (find-if (lambda (factor)
+                                    (or (eq factor var)
+                                        (and (mexptp factor)
+                                             (eq (cadr factor) var))))
+                                  (cdr d)))))
+               (if (not zero-root)
+                   (ratexpand1 n d)
+                   (let ((other-factors (remove zero-root (cdr d)
+                                                :test #'eq :count 1)))
+                     (m* (sratexpnd n (m*l other-factors))
+                         `((mexpt) ,zero-root -1)))))))))
 
 ; is a sum with index and bounds from psp2form
 (defun psp2formp (exp)
@@ -220,43 +263,77 @@
        (eq (cadddr exp) 0)
        (eq (cadr (cdddr exp)) '$inf)))
 
-; turns (%sum ...) + (%sum ...) + (%sum ...)
-; into   %sum ... + ... + ...
+;; A stripped down version of (sumcontract (intosum EXP)). Coalesce occurrences
+;; of ((%SUM) <FOO> *INDEX 0 '$INF), which are allowed to start with
+;; multiplicative constants. So something like
+;;
+;;    sum(a(i), i, 0, inf) + 2*sum(b(i), i, 0, inf) + c
+;;
+;; turns into
+;;
+;;    sum(a(i) + 2*b(i), i, 0, inf) + c
+;;
+;; This is good enough to collect up the multiple infinite sums that you get
+;; when expanding a power series by partial fractions. It's (obviously) not
+;; clever enough to rename index variables or adjust ranges.
+
 (defun psp2foldsum (exp)
-  (and $verbose
-       (prog2 (mtell (intl:gettext "powerseries: preparing to fold sums~%"))
-	   (show-exp exp)))
-  (if (and (eq (caar exp) 'mplus)
-	   (every #'(lambda (e) 
-		      (or (psp2formp e)
-			  (and (mtimesp e)
-			       (psp2formp (caddr e)))))
-		  (cdr exp)))
-      (list '(%sum) (m+l (mapcar #'(lambda (e)
-				     (if (eq (caar e) 'mtimes) 
-					 (m* (cadr e) (cadr (caddr e)))
-				       (cadr e)))
-				 (cdr exp)))
-	    *index 0 '$inf)
-    exp))
+  (when $verbose
+    (mtell (intl:gettext "powerseries: preparing to fold sums~%"))
+    (show-exp exp))
+
+  (multiple-value-bind (sums others)
+      (partition-by (lambda (e)
+                      (or (psp2formp e)
+                          (and (mtimesp e)
+                               (psp2formp (caddr e)))))
+                    (cdr exp))
+    (if (null sums)
+        ;; If there were no sums, just return the original expression.
+        exp
+        ;; Otherwise contract the sums
+        (let ((contracted
+               (list '(%sum)
+                     (m+l (mapcar (lambda (e)
+                                    (if (eq (caar e) 'mtimes)
+                                        (m* (cadr e) (cadr (caddr e)))
+                                        (cadr e)))
+                                  sums))
+                     *index 0 '$inf)))
+          (if (null others)
+              contracted
+              (m+l (cons contracted others)))))))
 
 ; solve returns a list: (soln mult soln mult ...)
 ; distinct-nonzero-roots-p returns true if every
 ;  soln is not nonzero and every mult is 1
 (defun distinct-nonzero-roots-p (roots)
   (or (null roots)
-      (and (not (zerop1 (caddar roots)))	; root must not be zero
-	   (eq 1 (cadr roots))			; multiplicity of root must be one
-	   (distinct-nonzero-roots-p (cddr roots)))))
+      (and
+       ;; Check all roots have multiplicity one and are nonzero.
+       (loop
+          for (root mult) on roots by #'cddr
+          always (and (eql 1 mult)
+                      (eq ($sign `((mabs) ,(caddr root))) '$pos)))
+
+       ;; Now check that the roots are genuinely different from one another
+       ;; (eg. if I have two symbolic roots, A and B, then I don't need to know
+       ;; the values of A and B, but I do need to know that they aren't equal)
+       (loop
+          for rest on roots by #'cddr
+          always (loop
+                    for b in (cddr rest) by #'cddr
+                    with a = (car rest)
+                    always (eq '$pos
+                               ($sign `((mabs) ,(m- (caddr b) (caddr a))))))))))
 
 ; returns t if polynomial poly in variable var has all distinct roots
 (defun has-distinct-nonzero-roots-p (poly var)
   (let ((*roots nil)
 	(*failures nil))
     (solve poly var 1)
-    (cond (*failures nil)
-	  ((distinct-nonzero-roots-p *roots) t)
-	  (t nil))))
+    (and (not *failures)
+         (distinct-nonzero-roots-p *roots))))
 
 ; Rational Expansion Theorem for Distinct Roots
 ; Graham, Knuth, Patashnik, "Concrete Math" 2nd ed, p 340
@@ -289,28 +366,47 @@
 		     *index 0))
 	  (t (error "EXPAND-DISTINCT-ROOTS: roots are not distinct.~%")))))
 
-(defun ratexand1 (n d)
-  (and $verbose
-       (prog2 (mtell (intl:gettext "powerseries: attempt partial fraction expansion of "))
-	   (show-exp (list '(mquotient) n d))
-	 (terpri)))
-  (funcall #'(lambda (*ratexp) 
-	       (let ((l ($partfrac (div* n d) var)))
-					 (cond ((eq (caar l) 'mplus)
-						(and $verbose
-			     (prog2 (mtell (intl:gettext "which is ~%"))
-				 (show-exp l)))
-			(psp2foldsum
-			 (m+l (mapcar #'ratexp
-				      (cdr l)))))
-		       ((poly? n var)
-			(and $verbose
-			     (mtell (intl:gettext "powerseries: partial fraction expansion failed, expanding denominator only.~%")))
-			(m* n (ratexp (m// 1 d))))
-					       (t (throw 'psex
-						    '(err (mtext)
-				       (intl:gettext "powerseries: partial fraction expansion failed")))))))
-	   t))
+;; Try to expand N/D as a power series in VAR about 0.
+;;
+;; Can safely assume that D has no zero roots (we remove them in SRATEXPND
+;; before calling this function).
+(defun ratexpand1 (n d)
+  (when $verbose
+    (mtell (intl:gettext
+            "powerseries: attempt partial fraction expansion of "))
+    (show-exp (list '(mquotient) n d))
+    (terpri))
+
+  ;; *RATEXP serves as a recursion guard: if SRATEXPND is about to call us
+  ;; *recursively, the flag will be set and it gives up instead.
+  (let* ((*ratexp t)
+         (fractions ($partfrac (div* n d) var)))
+    (cond
+      ;; If $PARTFRAC succeeds, it will return a sum of terms. In that case,
+      ;; expand each one (which we assume is going to be easier than what we
+      ;; started with) and try to fold the result into a single power series sum
+      ;; again.
+      ((mplusp fractions)
+       (when $verbose
+         (mtell (intl:gettext "which is ~%"))
+         (show-exp fractions))
+       (psp2foldsum (m+l (mapcar #'ratexp (cdr fractions)))))
+
+      ;; If that didn't work, maybe it's because the numerator messed things
+      ;; up. If we're really lucky, the numerator is actually a polynomial
+      ;; though. In that case, factor it out and do an expansion of 1/d on its
+      ;; own.
+      ((poly? n var)
+       (when $verbose
+         (mtell (intl:gettext
+                 "powerseries: partial fraction expansion failed, ~
+                  expanding denominator only.~%")))
+       (m* n (ratexp (m// 1 d))))
+
+      ;; If n is complicated, there's not really much we can do to make further
+      ;; progress, so give up and return a noun form.
+      (t (powerseries-expansion-error
+          (intl:gettext "Partial fraction expansion failed"))))))
 
 (defun sratsubst (gcd num den)
   (and $verbose
@@ -343,19 +439,59 @@
 	       (t (cons *n (exlist (cdr exp))))))
 	(t (exlist (cdr exp)))))
 
+;; Perform a binomial expansion of (c x^m + a)^n.
+;;
+;; If n isn't known to be an integer, we'd like to write a sum of terms
+;;
+;;    x^(mk) c^k a^(n-k) / ((n+1) beta(n-k+1, k+1))
+;;
+;; But we have to be a bit careful: it only works if c, x and a are
+;; nonzero. (Otherwise the simplifier quite reasonably simplifies each of the
+;; terms to zero, because the only nonzero term has the undefined term 0^0 in
+;; it).
+;;
+;; If we're not sure whether cx is zero, we can just split out the first term
+;; from the sum. If a is definitely zero, we can just write down a single
+;; term. If we're not sure about a, it's a bit more difficult: if we know that n
+;; is a positive integer, the sum is finite (and has at least two terms) and we
+;; can just split off the last term. Otherwise, give up.
 (defun srbinexpnd (ans)
-  (let ((n (cdr (assoc 'n ans :test #'eq)))
-	(a (cdr (assoc 'a ans :test #'eq)))
-	(m (cdr (assoc 'm ans :test #'eq)))
-	(c (cdr (assoc 'c ans :test #'eq))))
-    (cond ((and (integerp n) (minusp n))
-	   (srintegexpd (neg n) a m c))
-	  (t (list '(%sum)
-		   (m// (m* (m^ (m* c var) (m* m *index))
-			    (m^ a (m- n *index)))
-			(m* (list '($beta) (m- n (m1- *index)) (m1+ *index))
-			    (m1+ n)))
-		   *index 0 '$inf)))))
+  (alist-bind (n a m c cc x) ans
+    (m* cc
+        (if (and (integerp n) (minusp n))
+            (srintegexpd (neg n) a m c)
+            (let ((sgn-a ($sign (list '(mabs) a)))
+                  (sgn-cx ($sign `((mabs) ((mtimes) ,c ,x))))
+                  (sgn-n ($sign n))
+                  (general-term
+                   (m// (m* (m^ var (m* m *index))
+                            (m^ c *index)
+                            (m^ a (m- n *index)))
+                        (m* (list '($beta) (m- n (m1- *index)) (m1+ *index))
+                            (m1+ n)))))
+              (cond
+                ((eq sgn-n '$zero) 1)
+                ((eq sgn-cx '$zero) (m^ a n))
+                ((eq sgn-a '$zero) (m* (m^ c n) (m^ x (m* m n))))
+
+                ((and (eq sgn-a '$pos) (eq sgn-cx '$pos))
+                 (if (and ($featurep n '$integer)
+                          (memq sgn-n '($pos $pz)))
+                     `((%sum) ,general-term ,*index 0 ,n)
+                     `((%sum) ,general-term ,*index 0 $inf)))
+
+                ((eq sgn-a '$pos)
+                 (m+ (m^ a n) `((%sum) ,general-term ,*index 1 $inf)))
+
+                ((and ($featurep n '$integer) (eq sgn-n '$pos))
+                 (m+ `((%sum) ,general-term ,*index 0 ,(m1- n))
+                     (m* (m^ c n) (m^ x (m* n m)))))
+
+                (t
+                 (powerseries-expansion-error
+                  (intl:gettext
+                   "Couldn't expand binomial~%~M~%~
+                    as we didn't know which terms were nonzero.")))))))))
 
 (defun psp2form (coeff exp bas)
   (list '(%sum) (m* coeff (m^ var exp)) *index bas '$inf))
@@ -402,30 +538,133 @@
 
 (defun sp2trig (exp) (subst *index '*index (oldget (caar exp) 'sp2)))
 
+;; Take an expression, EXPR, and try to write it as a + b*VAR^c. On success,
+;; returns (VALUES A B C). On failure, raise a powerseries-expansion-error.
+;;
+;; One way to do this would be to call $EXPAND and look at the results, but that
+;; might be rather slow - if the expression is something like (1+x)^10, expand
+;; will take ages and won't help very much. Another approach would be to put it
+;; into CRE form, but that would be a mistake if the input was something like
+;; (1+x)^100...
+;;
+;; Instead, we're a bit stupider: we walk through the expression tree. If we see
+;; anything other than +,* and ^, we give up. If we find we have more than one
+;; different exponent in our terms, we give up.
+;;
+;; Obviously, there are always examples where this won't work, but $EXPAND will
+;; (something like (x+1)^2 - x^2, for example), but I'm assuming that this won't
+;; be something we encounter in practice.
+(defun split-two-term-poly (expr)
+  (cond
+    ((atom expr)
+     (if (eq expr var)
+         (values 0 1 1)
+         (values expr 0 1)))
+
+    ((freeof var expr)
+     (values expr 0 1))
+
+    ((eq (caar expr) 'mplus)
+     (let ((a 0) (b) (c))
+       (dolist (arg (cdr expr))
+         (multiple-value-bind (aa bb cc) (split-two-term-poly arg)
+           (setf a (m+ a aa))
+           (unless (eql bb 0)
+             (cond
+               ;; Is this the first nonzero power we've seen?
+               ((not b)
+                (setf b bb c cc))
+               ;; Is this another term with the same power as one we've seen
+               ;; already?
+               ((eql c cc)
+                (setf b (m+ b bb)))
+               ;; Otherwise, give up.
+               (t
+                (powerseries-expansion-error))))))
+       (values a b c)))
+
+    ((eq (caar expr) 'mtimes)
+     (let ((prod 1) a b c)
+       (dolist (arg (cdr expr))
+         (multiple-value-bind (aa bb cc) (split-two-term-poly arg)
+           (cond
+             ((eql bb 0)
+              (setf prod (m* prod aa)))
+             ((not a)
+              (setf a aa b bb c cc))
+             ;; This is the second term (a + b*x^c), so we know we'll end up
+             ;; with mixed terms. (We don't try to spot e.g. (1-x)*(1+x))
+             (t
+              (powerseries-expansion-error)))))
+
+       (if a
+           (values (m* a prod) (m* b prod) c)
+           (values prod 0 1))))
+
+    ((eq (caar expr) 'mexpt)
+     ;; We know that EXPR isn't free of VAR. Check that VAR isn't in the
+     ;; exponent.
+     (unless (freeof var (caddr expr))
+       (powerseries-expansion-error))
+     (multiple-value-bind (a b c) (split-two-term-poly (cadr expr))
+       (cond
+         ((eql a 0)
+          (values 0 (m^ b (caddr expr)) (m* c (caddr expr))))
+         ((eql b 0)
+          (values (m^ a (caddr expr)) 0 1))
+         (t
+          (powerseries-expansion-error)))))
+
+    (t
+     (powerseries-expansion-error))))
+
+;; Try to expand log(e) using the series for log(1+x).
+;;
+;; The basic idea is that if a is nonzero then
+;;
+;;   log(a + b*x^k) = log(a*(1 + b/a*x^k))
+;;                  = log(a) + log(1 + b/a*x^k)
+;;
+;; and we know a series for that.
 (defun sp2log (e)
-  (funcall #'(lambda (exp *a *n)
-	       (cond ((or (atom e) (free e var)) (list '(%log) e))
-		     ((null (smono exp var)) (throw 'psex nil))
-		     ((or (and (numberp *a)
-			       (minusp *a)
-			       (setq *a (- *a)))
-			  (and (mtimesp *a)
-			       (numberp (cadr *a))
-			       (minusp (cadr *a))
-			       (setq *a (simptimes
-					 (list* (car *a) (- (cadr *a)) (cddr *a))
-					 1 t))))
-		      (list '(%sum)
-			    (m* -1
-				(m^ (m* (m^ var *n) *a) *index)
-				(m^ *index -1))
-			    *index 1 '$inf))
-		     (t (list '(%sum)
-			      (m* -1
-				  (m^ (m* -1 *a (m^ var *n)) *index)
-				  (m^ *index -1))
-			      *index 1 '$inf))))
-	   (m- e 1) nil nil))
+  (cond
+    ((or (atom e) (free e var))
+     (list '(%log) e))
+    (t
+     (multiple-value-bind (a b k)
+         (split-two-term-poly (specdisrep e))
+       ;; If a is zero, we can't expand
+       (unless (eq '$positive
+                   (asksign (list '(mabs) a)))
+         (powerseries-expansion-error))
+
+       (let* ((coeff (m* b (m^ a -1)))
+              (coeff-sign ($sign coeff))
+              (negate-coeff-p))
+         ;; If we know that the coefficient is not positive, switch the series
+         ;; around (which gets rid of some ugly minus signs). If we're not sure,
+         ;; but the cofficient "looks negative" (so is something like -7*k),
+         ;; switch it around too.
+         (cond
+           ((member coeff-sign '($neg $nz))
+            (setf negate-coeff-p t
+                  coeff (m- coeff)))
+           ((and (not (member coeff-sign '($pos $pz)))
+                 (mtimesp coeff)
+                 (numberp (cadr coeff))
+                 (minusp (cadr coeff)))
+            (setq negate-coeff-p t
+                  coeff (simptimes
+                         (list* (car coeff) (- (cadr coeff)) (cddr coeff))
+                         1 t))))
+         (list '(%sum)
+               (m* -1
+                   (m^ (if negate-coeff-p
+                           (m* coeff (m^ var k))
+                           (m* -1 coeff (m^ var k)))
+                       *index)
+                   (m^ *index -1))
+               *index 1 '$inf))))))
 
 (defun sp2expt (exp)
   (let ((base (cadr exp))
@@ -462,7 +701,7 @@
             (m*l (mapcar (lambda (summand) (sp2sub expansion summand))
                          (cdr power)))))))
 
-      (t (throw 'psex nil)))))
+      (t (powerseries-expansion-error)))))
 
 (defun dup (x %n)
   (if (= %n 1)
@@ -483,7 +722,7 @@
           do
             (cond
               ((not (typep order '(integer 0)))
-               (throw 'psex nil))
+               (powerseries-expansion-error))
 
               ((not (eq y var))
                (setf remaining-derivs (list* order y remaining-derivs)))
@@ -579,6 +818,45 @@
 		 (t (sinint exp var))))
 	  (t (sinint exp var)))))
 
+;; Substitute NEW for OLD in the expression tree EXPRESSION. This is a bit like
+;; MAXIMA-SUBSTITUTE, except it assumes OLD is a symbol. But the important bit
+;; is that it is bright enough to avoid substituting for bound variables in
+;; %INTEGRATE and %AT forms: even though the symbol might have the same name,
+;; it's thought of as a logically different variable.
+;;
+;; The function returns two values: the new expression and a flag that says
+;; whether that expression has changed. If the flag is true on a recursive call,
+;; int-diff-substitute resimplifies the result.
+(defun int-diff-substitute (new old expression)
+  (cond
+    ((eq expression old) (values new t))
+    ((atom expression) (values expression nil))
+    (t
+     (let ((op (caar expression))
+           (args (cdr expression)))
+       (if (or (and (eq op '%integrate) (eq old (second args)))
+               (and (eq op '%at)
+                    (not (atom (second args)))
+                    (if (eq (caar (second args)) 'mlist)
+                      ;; (second args) looks like ((mlist) ((mequal) ...) ...)
+                      (memq old (mapcar #'second (rest (second args))))
+                      ;; (second args) looks like ((mequal) ...)
+                      (eq old (second (second args))))))
+           (values expression nil)
+           (let* ((some-changed-p nil)
+                  (new-args
+                   (mapcar (lambda (x)
+                             (multiple-value-bind (new-val changed-p)
+                                 (int-diff-substitute new old x)
+                               (when changed-p
+                                 (setf some-changed-p t))
+                               new-val))
+                           (cdr expression))))
+             (if (not some-changed-p)
+                 (values expression nil)
+                 (values
+                  (simplifya (cons (list op) new-args) nil) t))))))))
+
 ;; Expand a definite integral, integrate(exp, v, lo, hi).
 (defun sp2integ2 (exp v lo hi)
   ;; Deal with the case where the integration variable is also the expansion
@@ -590,10 +868,9 @@
           exp (subst v var exp)))
 
   (when (boundp '*sp2integ-recursion-guard*)
-    (throw 'psex
-      (list 'err '(mtext)
-            "Recursion when trying to expand the definite integral "
-            (out-of (symbol-value '*sp2integ-recursion-guard*)))))
+    (powerseries-expansion-error
+     (intl:gettext "Recursion when trying to expand the definite integral: ~M")
+     (out-of (symbol-value '*sp2integ-recursion-guard*))))
 
   (cond
     ((not (and (free lo var) (free hi var)))
@@ -607,10 +884,9 @@
        (declare (special *sp2integ-recursion-guard*))
 
        (unless (and (smono lo-exp var) (smono hi-exp var))
-         (throw 'psex
-           (list 'err '(mtext)
-                 "Endpoints of definite integral " (out-of exp)
-                 " are not monomials in the expansion variable.")))
+         (powerseries-expansion-error
+          (intl:gettext "Endpoints of definite integral ~M are not monomials in ~
+                         the expansion variable.") (out-of exp)))
 
        ;; Since this is a formal powerseries calculation, we needn't concern
        ;; ourselves with radii of convergence here. Just expand the integrand
@@ -622,8 +898,8 @@
          ;; monomials in VAR. The difference of the two expressions isn't quite
          ;; of the right form, but hopefully it's close enough for any other
          ;; code that uses it.
-         (m- (maxima-substitute hi-exp v anti-derivative)
-             (maxima-substitute lo-exp v anti-derivative)))))
+         (m- (int-diff-substitute hi-exp v anti-derivative)
+             (int-diff-substitute lo-exp v anti-derivative)))))
 
     ((free exp var)
      (list '(%integrate) (subst var v exp) var lo hi))
