@@ -27,21 +27,28 @@
 
 (defun maxima-getenv (envvar)
   #+gcl     (si::getenv envvar)
+  #+ecl     (si::getenv envvar)
   #+allegro (system:getenv envvar)
   #+(or cmu scl) (cdr (assoc envvar ext:*environment-list* :test #'string=))
   #+sbcl    (sb-ext:posix-getenv envvar)
   #+clisp   (ext:getenv envvar)
-  #+mcl     (ccl::getenv envvar)
+  #+(or openmcl mcl)     (ccl::getenv envvar)
+  #+lispworks (hcl:getenv envvar)
+  #+abcl (ext:getenv envvar)
   )
 
 ;; CMUCL needs because when maxima reaches EOF, it calls BYE, not $QUIT.
 
 (defun bye ()
   #+(or cmu scl clisp) (ext:quit)
-  #+sbcl           (sb-ext:quit)
-  #+allegro        (excl:exit)
-  #+mcl            (ccl:quit)
-  #+gcl            (lisp:quit)
+  #+sbcl               (sb-ext:quit)
+  #+allegro            (excl:exit 0 :quiet t)
+  #+(or mcl openmcl)   (ccl:quit)
+  #+gcl                (lisp:quit)
+  #+ecl                (si:quit)
+  #+lispworks          (lispworks:quit)
+  #+abcl               (cl-user::quit)
+  #+kcl                (lisp::bye)
   )
 
 
@@ -86,6 +93,17 @@
 	((atom l) (member l x :test #'eq))
 	(t (or (amongl x (car l)) (amongl x (cdr l)))))) 
 
+;;; Tests to see whether one tree is a subtree of another.
+;;;
+;;; Both arguments should be well-formed cons trees (so no cycles). If supplied,
+;;; TEST is used as an equality predicate.
+
+(defun subtree-p (branch tree &key (test #'eql))
+  (or (funcall test branch tree)
+      (and (not (atom tree))
+           (member branch tree
+                   :test (lambda (x y) (subtree-p x y :test test))))))
+
 ;;; Takes a list in "alist" form and converts it to one in
 ;;; "property list" form, i.e. ((A . B) (C . D)) --> (A B C D).
 ;;; All elements of the list better be conses.
@@ -109,9 +127,3 @@
 	 nil)
 	(t
 	 (putprop bas val sel))))
-
-;;; This is like the function SYMBOLCONC except that it binds base
-
-(defmfun concat n
-  (let ((*print-base* 10.))
-    (implode (mapcan #'exploden (listify n)))))
